void calc_generic_DMNRG(const Eigen & diagIp, const Eigen & diagI1, 
			unsigned int dimp, unsigned int dim1, 
			const Matrix & op1II, const Matrix & op2II,
			double sign, t_factor spinfactor,
			ChainSpectrum * cs,
			const Invar &Ip, const Invar &I1)
{
   const double Emin = getEmin(); // used in optimization
   const double Emax = getEmax();

   using namespace STAT;
   const bool boolT0opt = P::T0opt >= 0; // Enable T=0 optimizations
   const unsigned int T0opt = 
     (boolT0opt ? P::T0opt : 0); // index limit in for loops over
                                 // states

   const Matrix & rhoNIp = rho[Ip]; // hand optimised out of the loops
   const Matrix & rhoNI1 = rho[I1];

   if (!LAST_ITERATION()) {
      my_assert(rhoNIp.size1() == dimp);
      my_assert(rhoNI1.size1() == dim1);
   } else {
      my_assert(rhoNIp.size1() >= dimp);
      my_assert(rhoNI1.size1() >= dim1);
   }
   
   // OPTIMIZATION NOTE: the inner loop should involve the last index.
   for (unsigned int rm = 0; rm < dimp; rm++) {
      const t_eigen Em = diagIp.value(rm);
          
      for (unsigned int rj = 0; rj < dim1; rj++) {
	 if (boolT0opt && rm > T0opt && rj > T0opt) // does not contribute at T=0
	   continue;
	 
	 const t_eigen Ej = diagI1.value(rj);
            
	 DELTA d;
	 d.energy = Ej - Em;
	 
	 const double absE = abs(d.energy);
	 if (absE < Emin || absE > Emax) // does not contribute
	   continue;

	 // #### case A ####
	 weight_bucket sumA;

	 for (unsigned int ri = 0; ri < dimp; ri++) 
	   sumA += op2II(rj, ri) * rhoNIp(rm, ri); // rm <-> ri, rho symmetric
	 t_weight weightA = t_weight(sumA) * CONJ_ME( op1II(rj, rm) );
	 
	 // #### case B ####
	 weight_bucket sumB;

	 for (unsigned int ri = 0; ri < dim1; ri++)
	   sumB += CONJ_ME( op1II(ri, rm) ) * rhoNI1(rj, ri);

	 t_weight weightB = sumB;
	 weightB *= op2II(rj, rm);
	 d.weight = spinfactor * (weightA + (-sign) * weightB);

	 cs->add(scale * d.energy, d.weight);
      } 
   } 
}
